@@ -46,6 +46,7 @@ enum {
 	R_ASM_SYNTAX_NONE = 0,
 	R_ASM_SYNTAX_INTEL,
 	R_ASM_SYNTAX_ATT,
+	R_ASM_SYNTAX_MASM,
 	R_ASM_SYNTAX_REGNUM, // alias for capstone's NOREGNAME
 	R_ASM_SYNTAX_JZ, // hack to use jz instead of je on x86
 };
@@ -101,6 +102,8 @@ typedef struct r_asm_t {
 	RSyscall *syscall;
 	RNum *num;
 	char *features;
+	int invhex; // invalid instructions displayed in hex
+	int pcalign;
 } RAsm;
 
 typedef int (*RAsmModifyCallback)(RAsm *a, ut8 *buf, int field, ut64 val);
@@ -113,8 +116,8 @@ typedef struct r_asm_plugin_t {
 	char *license;
 	void *user; // user data pointer
 	int bits;
-	int (*init)(void *user);
-	int (*fini)(void *user);
+	bool (*init)(void *user);
+	bool (*fini)(void *user);
 	int (*disassemble)(RAsm *a, RAsmOp *op, const ut8 *buf, int len);
 	int (*assemble)(RAsm *a, RAsmOp *op, const char *buf);
 	RAsmModifyCallback modify;
@@ -152,7 +155,7 @@ R_API RList* r_asm_get_plugins(RAsm *a);
 /* code.c */
 R_API RAsmCode *r_asm_code_new(void);
 R_API void* r_asm_code_free(RAsmCode *acode);
-R_API int r_asm_code_set_equ (RAsmCode *code, const char *key, const char *value);
+R_API bool r_asm_code_set_equ (RAsmCode *code, const char *key, const char *value);
 R_API char *r_asm_code_equ_replace (RAsmCode *code, char *str);
 
 // accessors, to make bindings happy
@@ -174,20 +177,20 @@ extern RAsmPlugin r_asm_plugin_x86_nasm;
 extern RAsmPlugin r_asm_plugin_x86_cs;
 extern RAsmPlugin r_asm_plugin_arm_gnu;
 extern RAsmPlugin r_asm_plugin_arm_cs;
+extern RAsmPlugin r_asm_plugin_arm_as;
 extern RAsmPlugin r_asm_plugin_armthumb;
 extern RAsmPlugin r_asm_plugin_arm_winedbg;
 extern RAsmPlugin r_asm_plugin_csr;
-extern RAsmPlugin r_asm_plugin_m68k;
 extern RAsmPlugin r_asm_plugin_ppc_gnu;
 extern RAsmPlugin r_asm_plugin_ppc_cs;
 extern RAsmPlugin r_asm_plugin_sparc_gnu;
 extern RAsmPlugin r_asm_plugin_avr;
 extern RAsmPlugin r_asm_plugin_dalvik;
-extern RAsmPlugin r_asm_plugin_msil;
 extern RAsmPlugin r_asm_plugin_sh;
 extern RAsmPlugin r_asm_plugin_z80;
 extern RAsmPlugin r_asm_plugin_i8080;
 extern RAsmPlugin r_asm_plugin_m68k;
+extern RAsmPlugin r_asm_plugin_m68k_cs;
 extern RAsmPlugin r_asm_plugin_arc;
 extern RAsmPlugin r_asm_plugin_rar;
 extern RAsmPlugin r_asm_plugin_dcpu16;
@@ -214,6 +217,14 @@ extern RAsmPlugin r_asm_plugin_cris_gnu;
 extern RAsmPlugin r_asm_plugin_z80_cr;
 extern RAsmPlugin r_asm_plugin_lh5801;
 extern RAsmPlugin r_asm_plugin_hppa_gnu;
+extern RAsmPlugin r_asm_plugin_v810;
+extern RAsmPlugin r_asm_plugin_mcs96;
+extern RAsmPlugin r_asm_plugin_lm32;
+extern RAsmPlugin r_asm_plugin_riscv;
+extern RAsmPlugin r_asm_plugin_vax;
+extern RAsmPlugin r_asm_plugin_lanai_gnu;
+extern RAsmPlugin r_asm_plugin_xtensa;
+extern RAsmPlugin r_asm_plugin_tricore;
 #endif
 
 #ifdef __cplusplus
